import os
import sys
import subprocess
import shutil
import yaml
import eneel.adapters.postgres as postgres
import eneel.adapters.oracle as oracle
import eneel.adapters.sqlserver as sqlserver
import logging
logger = logging.getLogger('main_logger')


def create_relative_path(path_name):
    if not os.path.exists(path_name):
        os.makedirs(path_name)


def create_path(path_name):
    # Create path
    if not os.path.exists(path_name):
        os.makedirs(path_name)

    # Absolute path
    abs_temp_file_dir = os.path.abspath(path_name)
    return abs_temp_file_dir


def delete_path(path_name):
    if os.path.exists(path_name):
        try:
            shutil.rmtree(path_name)
        except:
            logger.debug("Could not delete directory")
            pass


def delete_file(file):
    if os.path.exists(file):
        os.remove(file)


def load_yaml_from_path(path):
    with open(path, 'r') as stream:
        try:
            return yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            logger.error(exc)


def load_yaml(stream):
    try:
        return yaml.safe_load(stream)
    except yaml.YAMLError as exc:
        logger.error(exc)


def load_file_contents(path, strip=True):
    if not os.path.exists(path):
        logger.error(path, ' not found')

    with open(path, 'rb') as handle:
        to_return = handle.read().decode('utf-8')

    if strip:
        to_return = to_return.strip()

    return to_return


def run_cmd(cmd):
    res = subprocess.run(cmd,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT, universal_newlines=True)
    if res.returncode == 0:
        return res.returncode, res.stdout
    elif res.stdout:
        return res.returncode, res.stdout
    else:
        return res.returncode, res.stderr






import logging
import os
import colorama

colorama.init()


def get_logger(project='eneel'):

    logger = logging.getLogger('main_logger')
    logger.setLevel('DEBUG')

    # Stream handler
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(logging.Formatter('%(message)s'))
    stream_handler.setLevel(logging.INFO)

    #stream_log_format = '%(asctime)s - %(levelname)s - %(message)s'

    #stream_handler = logging.StreamHandler()
    #stream_handler.setLevel(logging.INFO)
    #stream_formatter = logging.Formatter(stream_log_format)
    #stream_handler.setFormatter(stream_formatter)

    logger.addHandler(stream_handler)

    # File handler
    file_log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

    log_filename = "logs/" + project + ".log"
    os.makedirs(os.path.dirname(log_filename), exist_ok=True)

    file_handler = logging.FileHandler(log_filename)
    file_handler.setLevel(logging.DEBUG)
    file_formatter = logging.Formatter(file_log_format)
    file_handler.setFormatter(file_formatter)

    logger.addHandler(file_handler)

    return logger




import os
import sys
import eneel.utils as utils
import eneel.adapters.postgres as postgres
import eneel.adapters.oracle as oracle
import eneel.adapters.sqlserver as sqlserver

import logging
logger = logging.getLogger('main_logger')


def get_connections(connections_path=None, target=None):
    if not connections_path:
        connections_path = os.path.join(os.path.expanduser('~'), '.eneel/connections.yml')
    try:
        connections_file_contents = utils.load_file_contents(connections_path, strip=False)
        connections = utils.load_yaml(connections_file_contents)

        connections_dict = {}
        for conn in connections:
            name = conn
            type = connections[name]['type']
            read_only = connections[name].get('read_only')
            # If target is set from cli. Use tha ttarget. Else use from connections.yml
            if target:
                target_out = target
            else:
                target_out = connections[name]['target']

            credentials = connections[name]['outputs'][target_out]
            connection = {'name': conn, 'type': type, 'read_only': read_only, 'target': target_out,
                          'credentials': credentials}

            connections_dict[name] = connection
        return connections_dict
    except:
        logger.error("Could not load connections.yml")


def get_project(project):
    try:
        project_file_contents = utils.load_file_contents(project + '.yml', strip=False)
        project = utils.load_yaml(project_file_contents)

        return project
    except:
        logger.error(project + ".yml not found")
        sys.exit(-1)


def connection_from_config(connection_info):
    database = connection_info['credentials'].get('database')
    user = connection_info['credentials'].get('user')
    password = connection_info['credentials'].get('password')
    server = connection_info['credentials'].get('host')
    limit_rows = connection_info.get('credentials').get('limit_rows')
    table_where_clause = connection_info.get('credentials').get('table_where_clause')
    read_only = connection_info.get('read_only')
    type = connection_info.get('type')
    if connection_info.get('type') == 'oracle':
        server = connection_info['credentials'].get('host') + ':' + str(connection_info['credentials'].get('port'))
        return oracle.Database(server, user, password, database, limit_rows, table_where_clause, read_only)
    elif connection_info.get('type') == 'sqlserver':
        odbc_driver = connection_info['credentials'].get('driver')
        trusted_connection = connection_info['credentials'].get('trusted_connection')
        as_columnstore = connection_info.get('credentials').get('as_columnstore')
        return sqlserver.Database(odbc_driver, server, database, limit_rows, user, password, trusted_connection,
                                  as_columnstore, read_only)
    elif connection_info.get('type') == 'postgres':
        return postgres.Database(server, user, password, database, limit_rows, read_only)
    else:
        logger.error('source type not found')


class Connections:
    def __init__(self, connections_path=None, target=None):
        self._connections_path = connections_path
        if not self._connections_path:
            self._connections_path = os.path.join(os.path.expanduser('~'), '.eneel/connections.yml')

        self.target = target

        self.connections = get_connections()

    def __enter__(self):
        return self

    def get_connections(self):
        try:
            connections_file_contents = utils.load_file_contents(self._connections_path, strip=False)
            connections = utils.load_yaml(connections_file_contents)

            print(connections)

            connections_dict = {}
            for conn in connections:
                name = conn
                type = connections[name]['type']
                read_only = connections[name].get('read_only')

                print(self._target)

                if not self._target:
                    self._target = connections[name]['target']
                    print(self._target)
                credentials = connections[name]['outputs'][self._target]
                print(credentials)
                connection = {'name': conn, 'type': type, 'read_only': read_only, 'target': target,
                              'credentials': credentials}

                print(connection)

                connections_dict[name] = connection
                print(connections_dict)
            return connections_dict
        except:
            logger.error("Could not load connections.yml")


class Project:
    def __init__(self, project_name, connections):
        self.project_name = project_name

        # Get configurations
        self.connections = connections
        self.project_config = get_project(project_name)

        self.source_name = self.project_config['source']
        self.target_name = self.project_config['target']

        self.source_conninfo = connections[self.source_name]
        self.target_conninfo = connections[self.target_name]

        self.project = self.project_config.copy()
        del self.project['schemas']

        self.temp_path = self.project.get('temp_path', 'temp')
        self.temp_path = os.path.join(self.temp_path , project_name)
        self.keep_tempfiles = self.project.get('keep_tempfiles', False)

        self.workers = self.project.get('parallel_loads', 1)

        self.loads = self.get_loads()

        self.num_tables_to_load = len(self.loads)

    def __enter__(self):
        return self

    def get_loads(self):
        # Lists of load settings
        load_orders = []
        project_names = []
        source_conninfos = []
        target_conninfos = []
        projects = []
        schemas = []
        tables = []
        temp_paths = []

        # Populate load settings
        order_num = 1
        for schema_config in self.project_config['schemas']:
            schema = schema_config.copy()
            del schema['tables']
            for table in schema_config['tables']:
                source_conninfo_item = self.source_conninfo
                target_conninfo_item = self.target_conninfo
                project_item = self.project
                schema_item = schema
                table_item = table

                load_orders.append(order_num)
                order_num += 1
                project_names.append(self.project_name)
                source_conninfos.append(source_conninfo_item)
                target_conninfos.append(target_conninfo_item)
                projects.append(project_item)
                schemas.append(schema_item)
                tables.append(table_item)
                temp_paths.append(self.temp_path)

            # Number of loads
            num_tables_to_load = len(tables)

            num_tables_to_loads = []
            for i in range(num_tables_to_load):
                num_tables_to_loads.append(num_tables_to_load)

        loads = [{'load_order': load_order,
                  'num_tables_to_load': num_tables_to_load,
                  'project_name': project_name,
                  'source_conninfo': source_conninfo,
                  'target_conninfo': target_conninfo,
                  'project': project,
                  'schema': schema,
                  'table': table,
                  'temp_path': temp_path
                  }
                 for load_order,
                     num_tables_to_load,
                     project_name,
                     source_conninfo,
                     target_conninfo,
                     project,
                     schema,
                     table,
                     temp_path in zip(load_orders,
                                      num_tables_to_loads,
                                      project_names,
                                      source_conninfos,
                                      target_conninfos,
                                      projects,
                                      schemas,
                                      tables,
                                      temp_paths)]

        return loads




import os
import sys
import psycopg2
import psycopg2.extras
import eneel.printer as printer

import logging
logger = logging.getLogger('main_logger')


class Database:
    def __init__(self, server, user, password, database, limit_rows=None, read_only=False):
        try:
            conn_string = "host=" + server + " dbname=" + \
                          database + " user=" + user + " password=" + password
            self._server = server
            self._user = user
            self._password = password
            self._database = database
            self._dialect = "postgres"
            self._limit_rows = limit_rows
            self._read_only = read_only

            self._conn = psycopg2.connect(conn_string)
            self._conn.autocommit = True
            self._cursor = self._conn.cursor()
            logger.debug("Connection to postgres successful")
        except psycopg2.Error as e:
            logger.error(e)
            sys.exit(1)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.commit()
        self._conn.close()

    def close(self):
        self._conn.close()
        logger.debug("Connection closed")

    @property
    def connection(self):
        return self._conn

    @property
    def cursor(self):
        return self._cursor

    def commit(self):
        self.connection.commit()

    def execute(self, sql, params=None):
        try:
            return self.cursor.execute(sql, params or ())
        except psycopg2.Error as e:
            logger.error(e)

    def execute_many(self, sql, values):
        try:
            return self.cursor.executemany(sql, values)
        except psycopg2.Error as e:
            logger.error(e)

    def fetchall(self):
        try:
            return self.cursor.fetchall()
        except psycopg2.Error as e:
            logger.error(e)

    def fetchone(self):
        try:
            return self.cursor.fetchone()
        except psycopg2.Error as e:
            logger.error(e)

    def fetchmany(self,rows):
        try:
            return self.cursor.fetchmany(rows)
        except psycopg2.Error as e:
            logger.error(e)

    def query(self, sql, params=None):
        try:
            self.cursor.execute(sql, params or ())
            return self.fetchall()
        except psycopg2.Error as e:
            logger.error(e)

    def schemas(self):
        try:
            q = 'SELECT schema_name FROM information_schema.schemata'
            schemas = self.query(q)
            return [row[0] for row in schemas]
        except:
            logger.error("Failed getting schemas")

    def tables(self):
        try:
            q = "select table_schema || '.' || table_name from information_schema.tables"
            tables = self.query(q)
            return tables
        except:
            logger.error("Failed getting tables")

    def table_columns(self, schema, table):
        try:
            q = """
                SELECT  
                      ordinal_position,
                      column_name,
                      data_type,
                      character_maximum_length,
                      numeric_precision,
                      numeric_scale
                FROM information_schema.columns
                WHERE 
                    --table_schema + '.' + table_name = ?
                    table_schema = %s
                    and table_name = %s
                    order by ordinal_position
            """
            columns = self.query(q, [schema, table])
            return columns
        except:
            logger.error("Failed getting columns")

    def create_table_script(self, table_name):
        columns = self.table_columns(table_name)
        statement = 'CREATE TABLE ' + table_name + '( '
        for col in columns:
            if col[2]:
                statement = statement + col[0] + ' ' + col[1] + '(' + str(col[2]) + '), '
            else:
                statement = statement + col[0] + ' ' + col[1] + ', '
        statement = statement[:-2] + ')'
        return statement

    def check_table_exist(self, table_name):
        try:
            check_statement = """
            SELECT EXISTS (
            SELECT 1
            FROM   information_schema.tables 
            WHERE  table_schema || '.' || table_name = '"""
            check_statement += table_name + "')"
            exists = self.query(check_statement)
            return exists[0][0]
        except:
            logger.error("Failed checking table exist")

    def truncate_table(self, table_name):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            sql = "TRUNCATE TABLE " + table_name
            self.execute(sql)
            logger.debug("Table " + table_name + " truncated")
        except:
            logger.error("Failed truncating table")

    def create_schema(self, schema):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            if schema in self.schemas():
                logger.debug("Schema exists")
            else:
                create_statement = 'CREATE SCHEMA ' + schema
                self.execute(create_statement)
                logger.debug("Schema" + schema + " created")
        except:
            logger.error("Failed creating schema")

    def get_max_column_value(self, table_name, column):
        try:
            sql = "SELECT MAX(" + column + ")::text FROM " + table_name
            max_value = self.query(sql)
            return max_value[0][0]
        except:
            logger.debug("Failed getting max column value")

    def export_table(self, schema, table, columns, path, delimiter=',', replication_key=None, max_replication_key=None):
        try:
            # Generate SQL statement for extract
            select_stmt = "SELECT "

            # Add columns
            for col in columns:
                column_name = col[1]
                select_stmt += column_name + ", "
            select_stmt = select_stmt[:-2]

            select_stmt += " FROM " + schema + '.' + table

            # Add incremental where
            if replication_key:
                select_stmt += " WHERE " + replication_key + " > " + "'" + max_replication_key + "'"

            # Add limit
            if self._limit_rows:
                select_stmt += " FETCH FIRST " + str(self._limit_rows) + " ROW ONLY"
            logger.debug(select_stmt)

            # Generate file name
            file_name = self._database + "_" + schema + "_" + table + ".csv"
            file_path = os.path.join(path, file_name)

            # Create and run the cmd
            sql = "COPY (%s) TO STDIN WITH DELIMITER AS '%s'"
            file = open(file_path, "w")
            try:
                self.cursor.copy_expert(sql=sql % (select_stmt, delimiter), file=file)
            except psycopg2.Error as e:
                logger.error(e)

            row_count = self.cursor.rowcount

            logger.debug(str(row_count) + " records exported")

            return file_path, delimiter
        except:
            logger.error("Failed exporting table")

    def insert_from_table_and_drop(self, schema, to_table, from_table):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        to_schema_table = schema + "." + to_table
        from_schema_table = schema + "." + from_table
        try:
            self.execute("INSERT INTO " + to_schema_table + " SELECT * FROM  " + from_schema_table)
            self.execute("DROP TABLE " + from_schema_table)
        except:
            logger.error("Failed to insert_from_table_and_drop")

    def switch_tables(self, schema, old_table, new_table):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:

            old_schema_table = schema + "." + old_table
            new_schema_table = schema + "." + new_table
            delete_table = old_table + "_delete"
            delete_schema_table = schema + "." + delete_table

            if self.check_table_exist(old_schema_table):
                self.execute("ALTER TABLE " + old_schema_table + " RENAME TO " + delete_table)
                self.execute("ALTER TABLE " + new_schema_table + " RENAME TO " + old_table)
                self.execute("DROP TABLE " + delete_schema_table)
                logger.debug("Switched tables")
            else:
                self.execute("ALTER TABLE " + new_schema_table + " RENAME TO " + old_table)
                logger.debug("Renamed temp table")
        except:
            logger.error("Failed to switch tables")

    def import_table(self, schema, table, file, delimiter=','):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            schema_table = schema + '.' + table

            sql = "COPY %s FROM STDIN WITH DELIMITER AS '%s'"
            file = open(file, "r")

            try:
                self.cursor.copy_expert(sql=sql % (schema_table, delimiter), file=file)
            except psycopg2.Error as e:
                logger.error(e)
                return "ERROR", e

            row_count = str(self.cursor.rowcount)

            #logger.info(row_count+ " records imported")

            return "DONE", row_count

        except:
            logger.error("Failed importing table")
            return "ERROR", e

    def generate_create_table_ddl(self, schema, table, columns):
        try:
            create_table_sql = "CREATE TABLE " + schema + "." + table + "(\n"
            for col in columns:

                ordinal_position = col[0]
                column_name = col[1]
                data_type = col[2].lower()
                character_maximum_length = col[3]
                numeric_precision = col[4]
                numeric_scale = col[5]

                column = column_name + " " + data_type
                if "char" in data_type:
                    column += "("
                    if character_maximum_length == -1:
                        column += "max"
                    else:
                        column += str(character_maximum_length)
                    column += ")"
                elif "numeric" in data_type:
                    column += "(" + str(numeric_precision) + "," + str(numeric_scale) + ")"
                elif data_type == "USER-DEFINED":
                    column = column_name + " TEXT"
                elif data_type == "ARRAY":
                    column = column_name + " TEXT"
                create_table_sql += column + ", \n"
            create_table_sql = create_table_sql[:-3]
            create_table_sql += ")"

            return create_table_sql
        except:
            logger.error("Failed generating create table script")

    def create_table_from_columns(self, schema, table, columns):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            table_exists = self.query("SELECT * FROM INFORMATION_SCHEMA.TABLES "
                                        "WHERE TABLE_SCHEMA = %s AND "
                                        "TABLE_NAME = %s", [schema, table])
            #table_exists = self.fetchone()

            if table_exists:
                self.execute("DROP TABLE " + schema + "." + table)

            self.create_schema(schema)
            create_table_sql = self.generate_create_table_ddl(schema, table, columns)
            self.execute(create_table_sql)
            logger.debug("table created")
        except:
            logger.error("Failed create table from columns")



import os
import cx_Oracle
import sys
import eneel.utils as utils
import logging
logger = logging.getLogger('main_logger')


class Database:
    def __init__(self, server, user, password, database, limit_rows=None, table_where_clause=None, read_only=False):
        try:
            server_db = server + "/" + database
            conn_string = user + ", " + password + ", " + server + "/" + database
            self._server = server
            self._user = user
            self._password = password
            self._database = database
            self._server_db = server_db
            self._dialect = "oracle"
            self._limit_rows = limit_rows
            self._table_where_clause = table_where_clause
            self._read_only = read_only

            self._conn = cx_Oracle.connect(user, password, server_db)
            self._cursor = self._conn.cursor()
            logger.debug("Connection to oracle successful")
        except cx_Oracle.Error as e:
            logger.error(e)
            sys.exit(1)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.commit()
        self._conn.close()

    def close(self):
        self._conn.close()
        #logger.info("Connection closed")

    @property
    def connection(self):
        return self._conn

    @property
    def cursor(self):
        return self._cursor

    def commit(self):
        self.connection.commit()

    def execute(self, sql, params=None):
        try:
            return self.cursor.execute(sql, params or ())
        except cx_Oracle.Error as e:
            logger.error(e)

    def execute_many(self, sql, values):
        try:
            return self.cursor.executemany(sql, values)
        except cx_Oracle.Error as e:
            logger.error(e)

    def fetchall(self):
        try:
            return self.cursor.fetchall()
        except cx_Oracle.Error as e:
            logger.error(e)

    def fetchone(self):
        try:
            return self.cursor.fetchone()
        except cx_Oracle.Error as e:
            logger.error(e)

    def fetchmany(self,rows):
        try:
            return self.cursor.fetchmany(rows)
        except cx_Oracle.Error as e:
            logger.error(e)

    def query(self, sql, params=None):
        try:
            self.cursor.execute(sql, params or ())
            return self.fetchall()
        except cx_Oracle.Error as e:
            logger.error(e)

    def schemas(self):
        try:
            q = 'SELECT DISTINCT OWNER FROM ALL_TABLES'
            schemas = self.query(q)
            return schemas
        except:
            logger.error("Failed getting schemas")

    def tables(self):
        try:
            q = "select OWNER || '.' || TABLE_NAME from ALL_TABLES"
            tables = self.query(q)
            return tables
        except:
            logger.error("Failed getting tables")

    def table_columns(self, schema, table):
        try:
            q = """
                SELECT 
                      COLUMN_ID AS ordinal_position,
                      COLUMN_NAME AS column_name,
                      DATA_TYPE AS data_type,
                      DATA_LENGTH AS character_maximum_length,
                      DATA_PRECISION AS numeric_precision,
                      DATA_SCALE AS numeric_scale
                FROM all_tab_cols
                WHERE 
                    owner = :s
                    and table_name = :t
                    AND COLUMN_ID IS NOT NULL
                    order by COLUMN_ID
                    """
            columns = self.query(q, [schema, table])
            return columns
        except:
            logger.error("Failed getting columns")

    def create_table_script(self, table_name):
        try:
            columns = self.table_columns(table_name)
            statement = 'CREATE TABLE ' + table_name + '( '
            for col in columns:
                if col[2]:
                    statement = statement + col[0] + ' ' + col[1] + '(' + str(col[2]) + '), '
                else:
                    statement = statement + col[0] + ' ' + col[1] + ', '
            statement = statement[:-2] + ')'
            return statement
        except:
            logger.error("Failed checking table exist")

    def check_table_exist(self, table_name):
        try:
            check_statement = """
            SELECT 1
           FROM   ALL_TABLES 
           WHERE  OWNER || '.' || TABLE_NAME = '""" + table_name + "'"
            exists = self.query(check_statement)
            if exists:
                return True
            else:
                return False
        except:
            logger.error("Failed checking table exist")

    def export_table(self, schema, table, columns, path, delimiter='|', replication_key=None, max_replication_key=None):
        try:
            # Generate SQL statement for extract
            select_stmt = "SELECT "
            for col in columns[:-1]:
                column_name = col[1]
                select_stmt += "REPLACE(" + column_name + ",chr(0),'')" + " || '" + delimiter + "' || \n"
            last_column_name = columns[-1:][0][1]
            select_stmt += last_column_name
            select_stmt += ' FROM ' + schema + "." + table

            # Where-claues for incremental replication
            if replication_key:
                replication_where = replication_key + " > " + "'" + max_replication_key + "'"
            else:
                replication_where = None

            wheres = replication_where, self._table_where_clause
            wheres = [x for x in wheres if x is not None]
            if len(wheres) > 0:
                select_stmt += " WHERE " + wheres[0]
                for where in wheres[1:]:
                    select_stmt += " AND " + where
            #if replication_key or self._table_where_clause:
            #    select_stmt += " WHERE "
            #if replication_key and not self._table_where_clause:
            #    select_stmt += replication_key + " > " + "'" + max_replication_key + "'"

            if self._limit_rows:
                select_stmt += " FETCH FIRST " + str(self._limit_rows) + " ROW ONLY"

            select_stmt += ";\n"

            # Generate file name
            file_name = self._database + "_" + schema + "_" + table + ".csv"
            file_path = os.path.join(path, file_name)

            spool_cmd = """set markup csv on quote off
            set term off
            set echo off
            set trimspool on 
            set trimout on
            set feedback off
            Set serveroutput off
            set heading off
            set arraysize 5000
            SET LONG 100000
            spool """

            spool_cmd += file_path + '\n'
            spool_cmd += select_stmt
            spool_cmd += "spool off\n"
            spool_cmd += "exit"
            logger.debug(spool_cmd)

            sql_file = os.path.join(path, self._database + "_" + schema + "_" + table + ".sql")

            with open(sql_file, "w") as text_file:
                text_file.write(spool_cmd)

            cmd = "SET NLS_LANG=SWEDISH_SWEDEN.WE8ISO8859P1\n"
            cmd += "set NLS_NUMERIC_CHARACTERS=. \n"
            cmd += "set NLS_TIMESTAMP_TZ_FORMAT=YYYY-MM-DD HH24:MI:SS.FF\n"
            cmd += "sqlplus " + self._user + "/" + self._password + "@//" + self._server_db + " @" + sql_file

            logger.debug(cmd)
            cmd_file = os.path.join(path, self._database + "_" + schema + "_" + table + ".cmd")

            with open(cmd_file, "w") as text_file:
                text_file.write(cmd)

            cmd_code, cmd_message = utils.run_cmd(cmd_file)
            if cmd_code == 0:
                logger.debug(schema + '.' + table + " exported")
            else:
                logger.error("Error exportng " + schema + '.' + table + " : cmd_code: " + str(cmd_code) + " cmd_message: " + cmd_message)

            return file_path, delimiter
        except:
            logger.error("Failed exporting table")

    def generate_create_table_ddl(self, schema, table, columns):
        create_table_sql = "CREATE TABLE " + schema + "." + table + "(\n"
        for col in columns:
            column = col.column_name + " " + col.data_type
            if "char" in col.data_type:
                column += "("
                if col.character_maximum_length == -1:
                    column += "max"
                else:
                    column += str(col.character_maximum_length)
                column += ")"
            elif "numeric" in col.data_type:
                column += "(" + str(col.numeric_precision) + "," + str(col.numeric_scale) + ")"
            elif col.data_type == "USER-DEFINED":
                column = col.column_name + " TEXT"
            elif col.data_type == "ARRAY":
                column = col.column_name + " TEXT"
            create_table_sql += column + ", \n"
        create_table_sql = create_table_sql[:-3]
        create_table_sql += ")"

        return create_table_sql

    def create_table_from_columns(self, schema, table, columns):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        table_exists = self.check_table_exist(table)

        if table_exists:
            self.execute("DROP TABLE " + schema + "." + table)

        self.commit()

        create_table_sql = self.generate_create_table_ddl(schema, table, columns)
        self.execute(create_table_sql)




import colorama
import time

import logging
logger = logging.getLogger('main_logger')


COLOR_FG_RED = colorama.Fore.RED
COLOR_FG_GREEN = colorama.Fore.GREEN
COLOR_FG_YELLOW = colorama.Fore.YELLOW
COLOR_RESET_ALL = colorama.Style.RESET_ALL


def get_color(color):
    if color == 'red':
        return COLOR_FG_RED
    if color == 'green':
        return COLOR_FG_GREEN
    if color == 'yellow':
        return COLOR_FG_YELLOW
    else:
        return ''


PRINTER_WIDTH = 80


def get_timestamp():
    return time.strftime("%H:%M:%S")


def print_fancy_output_line(msg, status, index, total, execution_time=None,
                            truncate=False):
    if index is None or total is None:
        progress = ''
    else:
        progress = '{} of {} '.format(index, total)
    prefix = "{timestamp} | {progress}{message}".format(
        timestamp=get_timestamp(),
        progress=progress,
        message=msg)

    truncate_width = PRINTER_WIDTH - 3
    justified = prefix.ljust(PRINTER_WIDTH, ".")
    if truncate and len(justified) > truncate_width:
        justified = justified[:truncate_width] + '...'

    if execution_time is None:
        status_time = ""
    else:
        status_time = " in {execution_time:0.2f}s".format(
            execution_time=execution_time)

    if status == "OK":
        status = COLOR_FG_GREEN + status + COLOR_RESET_ALL

    status_txt = status

    output = "{justified} [{status}{status_time}]".format(
        justified=justified, status=status_txt, status_time=status_time)

    logger.info(output)


def print_output_line(msg):
    output = "{timestamp} | {message}".format(
        timestamp=get_timestamp(),
        message=msg)
    logger.info(output)


def print_load_line(index, total, status, table, rows=None, execution_time=None,
                    truncate=False, msg=''):
    if execution_time is not None and rows is not None:
        rows_per_sec = "at " + str(int(int(rows) / execution_time)) + " rows/sec"
    else:
        rows_per_sec = ""
    if index is None or total is None:
        progress = ''
    else:
        progress = '{} of {} '.format(index, total)
    prefix = "{timestamp} | {progress}{status} {table} {rows_per_sec}{msg}".format(
        timestamp=get_timestamp(),
        progress=progress,
        status=status,
        table=table,
        rows_per_sec=rows_per_sec,
        msg=msg)

    truncate_width = PRINTER_WIDTH - 3
    justified = prefix.ljust(PRINTER_WIDTH, ".")
    if truncate and len(justified) > truncate_width:
        justified = justified[:truncate_width] + '...'

    if execution_time is None:
        status_time = ""
    #elif rows is None:
    else:
        status_time = " in {execution_time:0.2f}s".format(
            execution_time=execution_time)
#    else:
#        rows_per_sec = str(int(int(rows) / execution_time))
#        status_time = " rows in {execution_time:0.2f}s ({rows_per_sec} /sec)".format(
#            execution_time=execution_time,
#            rows_per_sec=rows_per_sec)

    if status == "DONE":
        rows = get_color('green') + rows + COLOR_RESET_ALL
        output_txt = rows
    elif status == "WARN":
        rows = get_color('yellow') + rows + COLOR_RESET_ALL
        output_txt = rows
    elif status == "ERROR":
        output_txt = get_color('red') + status + COLOR_RESET_ALL
    else:
        output_txt = "RUN"

    output = "{justified} [{output}{status_time}]".format(
        justified=justified, output=output_txt, status_time=status_time)

    logger.info(output)


def print_msg(msg, color=None):
    if color:
        color = get_color(color)
        msg = color + msg + COLOR_RESET_ALL
    logger.info(msg)




import eneel.utils as utils
from concurrent.futures import ProcessPoolExecutor as Executor
import os
import eneel.printer as printer
import time
import eneel.config as config

import logging
logger = logging.getLogger('main_logger')


def run_project(project_name, connections_path=None, target=None):
    # Connections
    connections = config.Connections(connections_path, target)

    # Project
    project = config.Project(project_name, connections.connections)

    printer.print_msg('Running ' + project.project_name
                      + ' with ' + str(project.num_tables_to_load) + ' loadjobs from '
                      + project.source_name + ' to ' + project.target_name
                      )
    printer.print_msg('')

    workers = project.workers

    if project.num_tables_to_load < workers:
        workers = project.num_tables_to_load
    start_msg = "Start loading " + str(project.num_tables_to_load) + " tables with " + str(workers) + " parallel workers"
    printer.print_output_line(start_msg)

    job_start_time = time.time()

    # Execute parallel load
    load_results = []
    with Executor(max_workers=workers) as executor:
        for result in executor.map(run_load, project.loads):
            load_results.append(result)

    load_successes = 0
    load_warnings = 0
    load_errors = 0

    for load_result in load_results:
        if load_result == 'DONE':
            load_successes += 1
        if load_result == 'WARN':
            load_warnings += 1
        if load_result == 'ERROR':
            load_errors += 1

    # Clean up temp dir
    if not project.keep_tempfiles:
        utils.delete_path(project.temp_path)

    execution_time = time.time() - job_start_time

    status_time = " in {execution_time:0.2f}s".format(
        execution_time=execution_time)

    end_msg = "Finished loading " + str(project.num_tables_to_load) + " tables in " + status_time + ": " + \
                                                            str(load_successes) + " successfull, " + \
                                                            str(load_warnings) + " with warnings and " + \
                                                            str(load_errors) + " with errors"
    printer.print_output_line("")
    printer.print_output_line(end_msg)

    printer.print_msg("")
    if load_errors > 0:
        printer.print_msg("Completed with errors", "red")
    elif load_warnings > 0:
        printer.print_msg("Completed with warnings", "yellow")
    else:
        printer.print_msg("Completed successfully", "green")


def run_load(project_load):
    load_order = project_load.get('load_order')
    num_tables_to_load = project_load.get('num_tables_to_load')
    project_name = project_load.get('project_name')
    source_conninfo = project_load.get('source_conninfo')
    target_conninfo = project_load.get('target_conninfo')
    project = project_load.get('project')
    temp_path = project_load.get('temp_path')
    schema = project_load.get('schema')
    table = project_load.get('table')

    return_code = 'ERROR'

    load_start_time = time.time()

    import eneel.logger as logger
    logger = logger.get_logger(project_name)

    # Remove duplicated handler if any
    for handler in logger.handlers[2:]:
        logger.removeHandler(handler)

    source = config.connection_from_config(source_conninfo)
    target = config.connection_from_config(target_conninfo)

    # Delimiter
    csv_delimiter = project.get('csv_delimiter')

    # Schemas
    source_schema = schema.get('source_schema')
    target_schema = schema.get('target_schema')

    # Tables
    source_table = table.get('table_name')
    full_source_table = source_schema + '.' + source_table
    target_table = schema.get('table_prefix', "") + table.get('table_name') + schema.get('table_suffix', "")
    full_target_table = target_schema + '.' + target_table
    index = load_order
    total = num_tables_to_load

    if not source.check_table_exist(full_source_table):
        printer.print_load_line(index, total, "ERROR", full_source_table, msg="does not exist in source")
        return return_code

    # Temp table
    target_table_tmp = target_table + '_tmp'

    # Temp path for specific load
    temp_path_schema = os.path.join(temp_path, source_schema)
    temp_path_load = os.path.join(temp_path_schema, source_table)
    utils.create_path(temp_path_load)

    # Source column types to exclude
    source_columntypes_to_exclude = project.get('source_columntypes_to_exclude')
    if source_columntypes_to_exclude:
        source_columntypes_to_exclude = source_columntypes_to_exclude.lower().replace(" ", "").split(",")

    # Columns to load
    try:
        columns = source.table_columns(source_schema, source_table)
        if source_columntypes_to_exclude:
            columns_to_load = columns.copy()
            for col in columns:
                data_type = col[2].lower()
                if data_type in source_columntypes_to_exclude:
                    columns_to_load.remove(col)
            columns = columns_to_load
    except:
        logger.error("Could not determine columns to load")
        return return_code

    # Load type
    replication_method = table.get('replication_method')

    if not replication_method or replication_method == "FULL_TABLE":
        index = load_order
        total = num_tables_to_load
        return_code = "START"
        table_msg = full_source_table + " (" + "FULL_TABLE" + ")"
        printer.print_load_line(index, total, return_code, table_msg)

        # Export table
        try:
            file, delimiter = source.export_table(source_schema, source_table, columns, temp_path_load,
                                              csv_delimiter)

            # Create temp table
            target.create_table_from_columns(target_schema, target_table_tmp, columns)

            # Import into temp table
            return_code, import_row_count = target.import_table(target_schema, target_table_tmp, file, delimiter)

            # Switch tables
            target.switch_tables(target_schema, target_table, target_table_tmp)

        except:
            printer.print_load_line(index, total, "ERROR", full_source_table, msg="load ")
            return return_code

    elif replication_method == "INCREMENTAL":
        try:
            index = load_order
            total = num_tables_to_load
            return_code = "START"
            table_msg = full_source_table + " (" + replication_method + ")"
            printer.print_load_line(index, total, return_code, table_msg)

            replication_key = table.get('replication_key')
            if not replication_key:
                printer.print_load_line(index, total, "ERROR", full_source_table, msg="replication key not defined")
                return return_code
            if replication_key not in [column[1] for column in columns]:
                printer.print_load_line(index, total, "ERROR", full_source_table, msg="replication key not found in table")
                return return_code

            max_replication_key = target.get_max_column_value(full_target_table, replication_key)
            if not max_replication_key:
                # Full export
                file, delimiter = source.export_table(source_schema, source_table, columns, temp_path_load,
                                                      csv_delimiter)
                # Create temp table
                target.create_table_from_columns(target_schema, target_table_tmp, columns)
                # Import into temp table
                return_code, import_row_count = target.import_table(target_schema, target_table_tmp, file, delimiter)
                # Switch tables
                target.switch_tables(target_schema, target_table, target_table_tmp)

            else:
                # Export new rows
                file, delimiter = source.export_table(source_schema, source_table, columns, temp_path_load, csv_delimiter,
                                                      replication_key, max_replication_key)
                # Create temp table
                target.create_table_from_columns(target_schema, target_table_tmp, columns)
                # Import into temp table
                return_code, import_row_count = target.import_table(target_schema, target_table_tmp, file, delimiter)
                # Insert into and drop
                target.insert_from_table_and_drop(target_schema, target_table, target_table_tmp)

        except:
            printer.print_load_line(index, total, "ERROR", full_source_table, msg="load failed")
            return return_code

    else:
        printer.print_load_line(index, total, "ERROR", full_source_table, msg="replication_method not valid")
        return return_code

    # delete temp folder
    if not project.get('keep_tempfiles', False):
        utils.delete_path(temp_path_load)

    # Close connections
    source.close()
    target.close()

    rows = import_row_count
    execution_time = time.time() - load_start_time
    printer.print_load_line(index, total, return_code, full_source_table, rows, execution_time)

    return return_code




import eneel.load_runner as load_runner
import argparse


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('project', help='The name of the project (i.e my_project for project.yml)')
    parser.add_argument('--connections', help='Optinally add the full path to connections.yml')
#    parser.add_argument('--logdir', help='For not using the default log directory')
    args = parser.parse_args()

    if not args.project:
        #logger.error("You need to supply your project name. I.e my_project to use my_project.yml")
        print("You need to supply your project name. I.e my_project to use my_project.yml")

#    if not args.connections:
#        logger.error("You need to supply a path to your connections.yml")

    else:
        project_name =  args.project
        import eneel.logger as logger
        logger = logger.get_logger(project_name)
        import eneel.printer as printer
        #        logger.info("Connections config: " + args.connections)
        printer.print_msg('')
        printer.print_msg('Running eneel ')
        printer.print_msg('')
        logger.debug("Loading project: " + project_name)
        try:
            load_runner.run_project(project_name, args.connections)
        except KeyboardInterrupt:
            print("Interupted by user")


if __name__ == '__main__':
    main()




import os
import sys
import pyodbc
import eneel.utils as utils
import logging
logger = logging.getLogger('main_logger')


class Database:
    def __init__(self, driver, server, database, limit_rows=None, user=None, password=None,
                 trusted_connection=None, as_columnstore=False, read_only=False):
        try:
            conn_string = "DRIVER={" + driver + "};SERVER=" + server + ";DATABASE=" + \
                          database
            if trusted_connection:
                conn_string += ";trusted_connection=yes"
            else:
                conn_string += ";UID=" + user + ";PWD=" + password
            self._server = server
            self._user = user
            self._password = password
            self._database = database
            self._dialect = "sqlserver"
            self._limit_rows = limit_rows
            self._as_columnstore = as_columnstore
            self._trusted_connection = trusted_connection
            self._read_only = read_only

            self._conn = pyodbc.connect(conn_string, autocommit=True)
            self._cursor = self._conn.cursor()
            logger.debug("Connection to sqlserver successful")
        except pyodbc.Error as e:
            logger.error(e)
            sys.exit(1)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.commit()
        self._conn.close()

    def close(self):
        self._conn.close()
        logger.debug("Connection closed")

    @property
    def connection(self):
        return self._conn

    @property
    def cursor(self):
        return self._cursor

    def commit(self):
        self.connection.commit()

    def execute(self, sql, params=None):
        try:
            return self.cursor.execute(sql, params or ())
        except pyodbc.Error as e:
            logger.error(e)

    def execute_many(self, sql, values):
        try:
            return self.cursor.executemany(sql, values)
        except pyodbc.Error as e:
            logger.error(e)

    def fetchall(self):
        try:
            return self.cursor.fetchall()
        except pyodbc.Error as e:
            logger.error(e)

    def fetchone(self):
        try:
            return self.cursor.fetchone()
        except pyodbc.Error as e:
            logger.error(e)

    def fetchmany(self, rows):
        try:
            return self.cursor.fetchmany(rows)
        except pyodbc.Error as e:
            logger.error(e)

    def query(self, sql, params=None):
        try:
            self.cursor.execute(sql, params or ())
            return self.fetchall()
        except pyodbc.Error as e:
            logger.error(e)

    def schemas(self):
        try:
            q = 'SELECT schema_name FROM information_schema.schemata'
            schemas = self.query(q)
            return [row[0] for row in schemas]
        except:
            logger.error("Failed getting schemas")

    def tables(self):
        try:
            q = "select table_schema + '.' + table_name from information_schema.tables"
            tables = self.query(q)
            return tables
        except:
            logger.error("Failed getting tables")

    def table_columns(self, schema, table):
        try:
            q = """
                SELECT  
                      ordinal_position,
                      column_name,
                      data_type,
                      character_maximum_length,
                      numeric_precision,
                      numeric_scale
                FROM information_schema.columns
                WHERE 
                    --table_schema + '.' + table_name = ?
                    table_schema = ?
                    and table_name = ?
                    order by ordinal_position
            """
            columns = self.query(q, [schema, table])
            return columns
        except:
            logger.error("Failed getting columns")

    def check_table_exist(self, table_name):
        try:
            check_statement = """
            SELECT 1
            FROM   information_schema.tables 
            WHERE  table_schema + '.' + table_name = '"""
            check_statement += table_name + "'"
            exists = self.query(check_statement)
            if exists:
                return True
            else:
                return False
        except:
            logger.error("Failed checking table exist")

    def truncate_table(self, table_name):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            sql = "TRUNCATE TABLE " + table_name
            self.execute(sql)
            logger.debug("Table " + table_name + " truncated")
        except:
            logger.error("Failed truncating table")

    def create_schema(self, schema):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            if schema in self.schemas():
                pass
            else:
                create_statement = 'CREATE SCHEMA ' + schema
                self.execute(create_statement)
                logger.debug("Schema" + schema + " created")
        except:
            logger.error("Failed creating schema")

    def get_max_column_value(self, table_name, column):
        try:
            sql = "SELECT cast(MAX(" + column + ") as varchar(max)) FROM " + table_name
            max_value = self.query(sql)
            logger.debug("Max " + column + " is " + str(max_value[0][0]))
            return max_value[0][0]
        except:
            logger.debug("Failed getting max column value")

    def export_table(self, schema, table, columns, path, delimiter='|', replication_key=None, max_replication_key=None, codepage='1252'):
        try:
            # Generate SQL statement for extract
            select_stmt = '"SELECT '

            # Add limit
            if self._limit_rows:
                select_stmt += 'TOP ' + str(self._limit_rows) + ' '

            # Add columns
            for col in columns:
                column_name = col[1]
                select_stmt += column_name + ", "
            select_stmt = select_stmt[:-2]

            select_stmt += ' FROM [' + self._database + '].[' + schema + '].[' + table + ']' + ' WITH (NOLOCK)'

            # Add incremental where
            if replication_key:
                select_stmt += " WHERE " + replication_key + " > " + "'" + max_replication_key + "'"
            select_stmt += '"'
            logger.debug(select_stmt)

            # Generate file name
            file_name = self._database + '_' + schema + '_' + table + '.csv'
            file_path = os.path.join(path, file_name)

            # Generate bcp command
            bcp_out = "bcp " + select_stmt + " queryout " + \
                      file_path + " -t" + delimiter + " -c -C" + codepage + " -S" + self._server
            if self._trusted_connection:
                bcp_out += " -T"
            else:
                bcp_out += " -U" + self._user + " -P" + self._password

            logger.debug(bcp_out)

            cmd_code, cmd_message = utils.run_cmd(bcp_out)
            if cmd_code == 0:
                try:
                    return_message = cmd_message.splitlines()
                    num_rows = str(return_message[-3].split()[0])
                    timing = str(return_message[-1].split()[5])
                    average = str(return_message[-1].split()[8][1:-3])
                    logger.debug(table + ": " + num_rows + " rows exported, in " + timing + " ms. at an average of " + average + " rows per sec")
                except:
                    logger.warning(table + ": " + "Failed to parse sucessfull export cmd for")
                logger.debug(schema + '.' + table + " exported")
            else:
                logger.error("Error exportng " + schema + '.' + table + " :" + cmd_message)

            return file_path, delimiter
        except:
            logger.error("Failed exporting table")

    def insert_from_table_and_drop(self, schema, to_table, from_table):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        to_schema_table = schema + "." + to_table
        from_schema_table = schema + "." + from_table
        try:
            self.execute("INSERT INTO " + to_schema_table + " SELECT * FROM  " + from_schema_table)
            self.execute("DROP TABLE " + from_schema_table)
        except:
            logger.error("Failed to insert_from_table_and_drop")

    def switch_tables(self, schema, old_table, new_table):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:

            old_schema_table = schema + "." + old_table
            new_schema_table = schema + "." + new_table
            delete_table = old_table + "_delete"
            delete_schema_table = schema + "." + delete_table

            if self.check_table_exist(old_schema_table):
                self.execute("EXEC sp_rename '" + old_schema_table + "', '" + delete_table + "'")
                self.execute("EXEC sp_rename '" + new_schema_table + "', '" + old_table + "'")
                self.execute("DROP TABLE " + delete_schema_table)
                logger.debug("Switched tables")
            else:
                self.execute("EXEC sp_rename '" + new_schema_table + "', '" + old_table + "'")
                logger.debug("Renamed temp table")
        except:
            logger.error("Failed to switch tables")

    def import_table(self, schema, table, file, delimiter=',', codepage='1252'):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            # Import data
            bcp_in = "bcp [" + self._database + "].[" + schema + "].[" + table + "] in " + \
                     file + " -t" + delimiter + " -c -C" + codepage + " -U -b100000 -S" + self._server
            if self._trusted_connection:
                bcp_in += " -T"
            else:
                bcp_in += " -U" + self._user + " -P" + self._password

            logger.debug(bcp_in)
            cmd_code, cmd_message = utils.run_cmd(bcp_in)
            if cmd_code == 0:
                try:
                    return_message = cmd_message.splitlines()
                    row_count = str(return_message[-3].split()[0])
                    try:
                        if return_message[2].split()[0] == 'SQLState':
                            logger.debug(cmd_message)
                            return "WARN", row_count
                    except:
                        pass

                    return "DONE", row_count
                except:
                    logger.warning(table + ": " + "Failed to parse sucessfull import cmd")
                    return "DONE", None
            else:
                logger.debug("Error importing " + schema + "." + table + " :" + cmd_message)
        except:
            logger.error("Failed importing table")

    def generate_create_table_ddl(self, schema, table, columns):
        try:
            create_table_sql = "CREATE TABLE " + schema + "." + table + "(\n"
            for col in columns:

                ordinal_position = col[0]
                column_name = col[1]
                data_type = col[2].lower()
                character_maximum_length = col[3]
                numeric_precision = col[4]
                numeric_scale = col[5]

                if "char" in data_type:
                    db_data_type = "VARCHAR" + "("
                    if character_maximum_length == -1:
                        db_data_type += "max"
                    else:
                        db_data_type += str(character_maximum_length)
                    db_data_type += ")"
                elif "numeric" in data_type or "number" in data_type or "decimal" in data_type:
                    if numeric_scale:
                        db_data_type = "NUMERIC" + "(" + str(numeric_precision) + "," + str(numeric_scale) + ")"
                    elif numeric_precision:
                        db_data_type = "NUMERIC" + "(" + str(numeric_precision) + ")"
                    else:
                        db_data_type = "FLOAT"
                elif "time zone" in data_type:
                    db_data_type = "DATETIME2"
                elif "timestamp" in data_type:
                    db_data_type = "DATETIME2"
                elif "bool" in data_type:
                    db_data_type = "INT"
                elif "clob" in data_type:
                    db_data_type = "VARCHAR(MAX)"
                else:
                    db_data_type = data_type

                column = column_name + " " + db_data_type
                create_table_sql += column + ", \n"

            create_table_sql = create_table_sql[:-3]
            create_table_sql += ")"

            logger.debug(create_table_sql)

            return create_table_sql
        except:
            logger.error("Failed generating create table script")

    def create_table_from_columns(self, schema, table, columns):
        if self._read_only:
            sys.exit("This source is readonly. Terminating load run")
        try:
            schema_table = schema + "." + table
            self.execute("SELECT * FROM INFORMATION_SCHEMA.TABLES "
                                        "WHERE TABLE_SCHEMA = ? AND "
                                        "TABLE_NAME = ?", [schema, table])
            table_exists = self.fetchone()

            if table_exists:
                self.execute("DROP TABLE " + schema_table)
                logger.debug("Table: " + schema_table + " droped")

            self.create_schema(schema)
            create_table_sql = self.generate_create_table_ddl(schema, table, columns)
            self.execute(create_table_sql)
            logger.debug("Table: " + schema_table + " created")

            # Create Clustered Columnstore Index if set on connection
            if self._as_columnstore:
                try:
                    index_name = schema + "_" + table + "_cci"
                    self.execute("DROP INDEX IF EXISTS " + index_name + " ON " + schema_table)
                    self.execute("CREATE CLUSTERED COLUMNSTORE INDEX " + index_name + " ON " + schema_table)
                    logger.debug("Index: " + index_name + " created")
                except:
                    logger.error("Failed create columnstoreindex")
        except:
            logger.error("Failed create table from columns")




#!/usr/scripts/env python
from setuptools import find_packages
from distutils.core import setup

import os
eneel_path = os.path.join(os.path.expanduser('~'), '.eneel')

package_name = "eneel"
package_version = "0.1.1"
description = """A package for fast loading av relational data"""

setup(
    name=package_name,
    version=package_version,
    description=description,
    long_description=description,
    long_description_content_type='text/markdown',
    author="Mikael Ene",
    author_email="mikael.ene@gmail.com",
    url="https://github.com/mikaelene/eneel",
    packages=find_packages(),
    install_requires=[
        'pyodbc>=4.0.27',
        'psycopg2>=2.7.7',
        'cx-Oracle>=7.0.0',
        'PyYAML>=3.11',
        'colorama>=0.3.9',
    ],
    entry_points={
            'console_scripts': ['eneel=eneel.main:main'],
        },
    data_files = [
        (eneel_path, ['example_connections.yml']),
    ],
)
